/**	Author: Travis Scott (ID 1554515)
 * 	Date: 14/12/2012
 */

package uoa.tsco033.evolisa;

import java.util.ArrayList;
import java.util.List;

import android.os.Parcel;
import android.os.Parcelable;
import android.util.Log;

/**This class represents a polygon, one of the building blocks of a drawing
 * It stores a list of points defining the outline, and a paint object representing the colour 
 * @author tsco033
 */
public class DNAPolygon implements Parcelable
{
	private ArrayList<DNAPoint> points;
	private DNABrush brush;

	// ---------- Setup ----------//

	public DNAPolygon()
	{

	}

	public DNAPolygon(List<DNAPoint> points, DNABrush brush)
	{
		// Create a new list of points
		this.points = new ArrayList<DNAPoint>();
		
		// Clone the points and add them to the new list
		for (DNAPoint point : points)
		{
			this.points.add(point.clone());
		}
		
		// Clone the brush
		this.brush = brush.clone();
	}
	
	public DNAPolygon clone()
	{
		return new DNAPolygon(points, brush);
	}

	public void initialise()
	{
		points = new ArrayList<DNAPoint>();

		DNAPoint origin = new DNAPoint();
		origin.initialise();

		for (int i = 0; i < Settings.ActivePointsPerPolygonMin; i++)
		{
			DNAPoint point = new DNAPoint(Math.min(Math.max(0, origin.getX() + Tools.getRandomNumber(-3, 3)), Settings.ActiveMaxWidth), Math.min(Math.max(0, origin.getY() + Tools.getRandomNumber(-3, 3)), Settings.ActiveMaxHeight));
			points.add(point);
		}

		brush = new DNABrush();
		brush.initialise();
	}

	// ---------- Parcelable Interface ----------//

	@Override
	public int describeContents()
	{
		return 0;
	}

	@Override
	public void writeToParcel(Parcel destination, int flags)
	{
		// Store fields
		destination.writeParcelable(brush, 0);
		destination.writeTypedList(points);
	}

	public DNAPolygon(Parcel source)
	{
		// Restore fields
		brush = (DNABrush) source.readParcelable(null);
		points = source.createTypedArrayList(DNAPoint.DNAPointCreator);
	}

	public static final Parcelable.Creator<DNAPolygon> DNAPolygonCreator = new Parcelable.Creator<DNAPolygon>()
	{
		public DNAPolygon createFromParcel(Parcel source)
		{
			return new DNAPolygon(source);
		}

		public DNAPolygon[] newArray(int size)
		{
			return new DNAPolygon[size];
		}
	};

	// ---------- Mutation ----------//

	/**Performs mutation of the polygon 
	 * @param drawing
	 */
	public void mutate(DNADrawing drawing)
	{
		// Check if a point should be added
		if (Tools.willMutate(Settings.ActiveAddPointMutationRate))
		{
			addPoint(drawing);
		}
		// Check if a point should be removed 
		if (Tools.willMutate(Settings.ActiveRemovePointMutationRate))
		{
			removePoint(drawing);
		}

		// Mutate the brush
		brush.mutate(drawing);

		// Mutate each point
		for (DNAPoint point : points)
		{
			point.mutate(drawing);
		}
	}

	// ---------- Methods ----------//

	/**Returns number of points the polygon has	 * 
	 * @return
	 */
	public int pointCount()
	{
		int pointCount = 0;
		for (DNAPoint point : points)
		{
			pointCount++;
		}
		return pointCount;
	}

	/**Removes a point from the polygon 
	 * @param drawing
	 */
	private void removePoint(DNADrawing drawing)
	{
		// Check we have more than the minimum points required for the drawing
		if (points.size() > Settings.ActivePointsPerPolygonMin)
		{
			// Check the polygon has more than the minimum required points
			if (drawing.pointCount() > Settings.ActivePointsMin)
			{
				// Generate a random number
				int index = Tools.getRandomNumber(0, points.size() - 1);
				
				// Remove the point
				points.remove(index);
				
				// Flag the drawing as dirty
				drawing.setDirty();
			}
		}
	}

	/**Adds a point to the polygon 
	 * @param drawing
	 */
	private void addPoint(DNADrawing drawing)
	{
		// Check we have fewer than the maximum number of points allowed for the drawing
		if (points.size() < Settings.ActivePointsPerPolygonMax)
		{
			// Check we have fewer than the maximum number of points allowed for the polygon
			if (drawing.pointCount() < Settings.ActivePointsMax)
			{
				// Generate a random number
				int index = Tools.getRandomNumber(1, points.size() - 1);
				
				// Fetch the points on either side of the new point
				DNAPoint prev = points.get(index - 1);
				DNAPoint next = points.get(index);
				
				// Put the new point halfway between the neighbouring points
				DNAPoint newPoint = new DNAPoint((prev.getX() + next.getX()) / 2, (prev.getY() + next.getY()) / 2);
				points.add(index, newPoint);
				
				// Flag the drawing as dirty
				drawing.setDirty();
			}
		}
	}

	// ---------- Accessors ----------//

	/**Returns the ARGB colour of the pixel as an int[]
	 * @return
	 */
	public int[] getColor()
	{
		return brush.getColor();
	}

	/**Returns the list of points defining the polygon
	 * @return
	 */
	public ArrayList<DNAPoint> getPoints()
	{
		return points;
	}
}
